#!/usr/bin/perl


#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.


use Getopt::Std;
use vars qw( %opt );

### Get command line options ###

&getopt('foksLU', \%opt);

### If insufficient options, print usage###

if(! ($opt{'f'}) ) 
{
   print "\nUsage: $0\n\n";
   print "-f <file.fasta> input reads\n"; 
   print "-o <output_file> sequences of viroid candidates (default: input_file.size.step)\n";
   print "-k <number> the fragment size\n"; 
   print "-s <number> the step size\n";
   print "-L <number> the mininal length of input sequence (default = 21)\n";
   print "-U <number> the maximal length of input sequence (default = 10000000000)\n\n";
}

### Set global parameters ###
my $kmer = 21;
my $step = 1; 
my $minL = $kmer;
my $maxL = 10000000000;

my $inputfile  = $opt{'f'};
my $outputfile = "$inputfile"."_$kmer"."_$step";

##
if ($opt{'o'}) { $outputfile = $opt{'o'};}
if ($opt{'k'}) { $kmer = $opt{'k'};}
if ($opt{'s'}) { $step = $opt{'s'};} 

if ($opt{'L'}) { $minL = $opt{'L'};}
if ($opt{'U'}) { $maxL = $opt{'U'};}



#
if(-e $inputfile) 
{ 
  $inputset = &readINFILE($inputfile,$outputfile,$minL,$maxL,$kmer,$step);	
}
else 
{ 
  die "\nPlease input the file (fasta format)!\n\n"; 
}

#################################################################
#reading input file

sub readINFILE
{
   my ($ifile,$ioutfile,$iminl,$imaxl,$ikmer,$istep) = @_;
   
   
   print "Start reading and spliting $ifile\n";

   my $icount=0;
   undef my @reads;

   open(INDATA, $ifile) || die "could not open $ifile\n";
   open(OUTDATA, ">$ioutfile");

   while(my $line=<INDATA>)
	{  
      chomp($line);

	  if($line=~m/^>/ )
		{
		  if(@reads >= 1) 
			  {   
		         shift(@reads);
				 my $readSeq = join("",@reads);
				 unless(length($readSeq) >= $iminl && length($readSeq) <= $imaxl) { next; } 

				 for(my $i=0; $i <= length($readSeq); $i+=$istep)
				  {
                     my $fragSeq = substr($readSeq,$i,$ikmer);
					 unless(length($fragSeq)==$ikmer) { next; }
					 $icount++;
                     print OUTDATA ">vSeq$icount\n$fragSeq\n";
                  }		  
		      }
          @reads=();
		}
	  push(@reads,$line);
	}
   ###############################
   		         shift(@reads);
				 my $readSeq = join("",@reads);
				 if(length($readSeq) >= $iminl && length($readSeq) <= $imaxl) { 

				 for(my $i=0; $i <= length($readSeq); $i+=$istep)
				  {
                     my $fragSeq = substr($readSeq,$i,$ikmer);
					 unless(length($fragSeq)==$ikmer) { next; }
					 $icount++;
                     print OUTDATA ">vSeq$icount\n$fragSeq\n";
                  }	
				 }
   #######################################
   close INDATA; close OUTDATA;
  
   print "Total small fragments: $icount\n";
   return 1;
}
###############################################################################################################################
